import java.applet.*;
import java.lang.*;
import java.util.*;
import java.text.*;

public class STZ extends Applet
{
        boolean			debug;
        boolean			daylight_savings_time;
        int			gmt_offset;
        int			start_month;
        int			start_week;
        int			start_day;
        int			start_time;
        int			end_month;
        int			end_week;
        int			end_day;
        int			end_time;
        String			tz_id;
        SimpleTimeZone	stz;
        Calendar		cal;
        SimpleDateFormat	formatter;

/**
 * Information about this applet.
 */
public String getAppletInfo()
{
        return "Show the current time.";
}

/**
 * Initializes the applet.
 *
 */
public void init()
{
        super.init();

        // Set up a default calendar.
        cal = Calendar.getInstance();

        // Set up a specific time zone, optionally with daylight savings.
        if (daylight_savings_time) stz = new SimpleTimeZone (gmt_offset, tz_id, start_month, start_week, start_day, start_time, end_month, end_week, end_day, end_time);
        else stz = new SimpleTimeZone (gmt_offset, tz_id);
        // Set up a date and time (24 hr) formatter, using the time zone.
        formatter = new SimpleDateFormat ("EEE, MMM d, yyyy HH:mm:ss z (zzzz)");
        formatter.setTimeZone(stz);
}

/**
 * Starts the applet.
 *
 */
public void start()
{
        super.start();

        // Display the current date and time.
        System.out.println();
        System.out.print ("The local time is ");
        System.out.println (formatter.format (cal.getTime()));
}

private String millis_to_HH_MM(int millis)
{
        String HH_MM;
        int hrs = millis / 3600000;
        int mins = (millis - (hrs * 3600000)) / 60000;

        if (hrs < -9) HH_MM = "-" + -hrs;
        else if (hrs < 0) HH_MM = "-0" + -hrs;
        else if (hrs < 10) HH_MM = " 0" + hrs;
        else HH_MM = " " + hrs;
        HH_MM += ":";
        if (mins < 0) HH_MM += -mins;
        else if (mins > 0) HH_MM += mins;
        else HH_MM += "00";

        return HH_MM;
}

private int HH_MM_to_millis (String HH_MM)
{
        int millis;
        int sign = 1;
        int hrs = 0;
        int mins = 0;
        String temp;
        String source = HH_MM.substring (0);
        if (debug)
        {
                System.out.print ("In: ");
                System.out.println (source);
        }
        // Parse the optional sign.
        temp = source.substring (0, 1);
        if (temp.equals ("-"))
        {
                sign = -1;
                source = source.substring (1);
                if (debug)
                {
                        System.out.print ("Parsed a minus sign: ");
                }
        }
        else if (temp.equals ("+"))
        {
                sign = 1;
                source = source.substring (1);
                if (debug)
                {
                        System.out.print ("Parsed a plus sign: ");
                }
        }
        if (debug)
        {
                System.out.println (source);
        }

        // Parse the hours.
        if (debug)
        {
                System.out.print ("Parsing hours: ");
        }
        while (source.length() > 0 && !temp.equals (":"))
        {
                temp = source.substring (0, 1);
                if (debug)
                {
                        System.out.println (temp);
                }
                if (!temp.equals (":")) hrs = hrs * 10 + Integer.valueOf (temp).intValue();
                source = source.substring (1);
                if (debug)
                {
                        System.out.println (source);
                }
        }

        // Parse the minutes.
        if (debug)
        {
                System.out.print ("Parsing minutes: ");
        }
        while (source.length() > 0)
        {
                temp = source.substring (0, 1);
                if (debug)
                {
                        System.out.println (temp);
                }
                mins = mins * 10 + Integer.valueOf (temp).intValue();
                source = source.substring (1);
                if (debug)
                {
                        System.out.println (source);
                }
        }

        // Finish up the conversion.
        millis = sign * (hrs * 60 * 60 * 1000 + mins * 60 * 1000);
        if (debug)
        {
                System.out.print ("Parsing HH:MM finished. Millis == ");
                System.out.println (millis);
        }
        return millis;
}

/**
 * main entrypoint - starts the applet when it is run as an application
 * @param args java.lang.String[]
 */
public static void main(java.lang.String[] args)
{
        int arg_count = args.length;
        int next = 0;
        STZ applet = new STZ();

        if (arg_count > 0 && args [next].equalsIgnoreCase ("-D"))
        {
                applet.debug = true;
                arg_count--;
                next++;
        }
        else applet.debug = false;

        if (applet.debug) for (int i = 0; i < args.length; i++) System.out.println (i + ": " + args [i]);

        if (arg_count == 0) System.out.println ("No time zone parameters. Use 'JRE STZ -?' or 'JAVA STZ -?' for help.");
        else if (args [next].equals ("-?"))
        {
                next++;
                if (arg_count == 1)
                {
                        System.out.println ();
                        System.out.println ("Usage: 'JRE STZ time_zone_parameters' or  'JAVA STZ time_zone_parameters'.");
                        System.out.println ();
                        System.out.println ("Displays the current time in 24 hour format. If optional time zone parameters");
                        System.out.println ("are specified, then the current time in the specified time zone is displayed.");
                        System.out.println ("All of this presumes that your time zone is properly configured for use with");
                        System.out.println ("Java (see http://David.Holm.org/jstreet/javatime.html for more details). The");
                        System.out.println ("time zone parameters are: gmt_offset, tz_id, [start_month, start_week,");
                        System.out.println ("start_day, start_time, end_month, end_week, end_day, end_time].");
                        System.out.println ();
                        System.out.println ("Example #1: JAVA STZ -05:30,EST,3,-1,1,02:00,9,2,2,17:00' sets up a time zone");
                        System.out.println ("that is 5-1/2 hours behind GMT and uses daylight savings time starting at 02:00");
                        System.out.println ("on the last Sunday in March and ending at 17:00 on the 2nd Monday in September.");
                        System.out.println ();
                        System.out.println ("Example #2: 'JRE STZ 8:00 CTT' sets up a time zone that is 8 hours ahead of GMT");
                        System.out.println ("and does not use daylight savings time.");
                        System.out.println ();
                        System.out.println ("To obtain a list of valid time zone IDs: 'JRE STZ -? ID' or 'JAVA STZ -? ID'.");
                        System.out.println ();
                        System.out.println ("To obtain the time zone ID details: 'JRE STZ -? TZ' or 'JAVA STZ -? TZ'.");
                }
                else if (args [next].equalsIgnoreCase ("ID"))
                {
                        System.out.println ("The valid time zone IDs are:");
                        String[] IDs = TimeZone.getAvailableIDs();
                        for (int i = 0; i < IDs.length; i++) System.out.print (IDs [i] + " ");
                }
                else if (args [next].equalsIgnoreCase ("TZ"))
                {
                        System.out.println ("The details for the valid time zone IDs are:");
                        String[] IDs = TimeZone.getAvailableIDs();
                        for (int i = 0; i < IDs.length; i++)
                        {
                                System.out.print (IDs [i] + " ");
                                TimeZone tz = TimeZone.getTimeZone( IDs [i]);
                                System.out.print (applet.millis_to_HH_MM (tz.getRawOffset()));
                                if (tz.useDaylightTime()) System.out.print (" D");
                                else System.out.print ("  ");
                                System.out.print ("    ");
                        }
                }
                else System.out.println ("'" + args [next] + "' is not a valid help option.");
        }
        else if (arg_count == 2 || arg_count == 10)
        {
		boolean ok = true;
		try
		{
	                applet.gmt_offset = applet.HH_MM_to_millis (args [next]);
	                next++;
	                applet.tz_id = args [next];
	                next++;
	                if (arg_count == 2) applet.daylight_savings_time = false;
	                else
	                {
	                        String temp;
	                        applet.daylight_savings_time = true;
	                        applet.start_month = Integer.valueOf (args [next]).intValue() - 1;
	                        next++;
	                        applet.start_week = Integer.valueOf (args [next]).intValue();
	                        next++;
	                        applet.start_day = Integer.valueOf (args [next]).intValue();
	                        next++;
	                        applet.start_time = applet.HH_MM_to_millis (args [next].toString());
        	                next++;
	                        applet.end_month = Integer.valueOf (args [next]).intValue() - 1;
	                        next++;
	                        applet.end_week = Integer.valueOf (args [next]).intValue();
	                        next++;
	                        applet.end_day = Integer.valueOf (args [next]).intValue();
	                        next++;
	                        applet.end_time = applet.HH_MM_to_millis (args [next].toString());
	                        next++;
			}
                }
		catch (NumberFormatException x)
		{
			ok = false;
			System.out.println ();
			System.out.println ("Parameter " + (next + 1) + " (" + args [next] + ") is not a valid number.");
		}
		if (ok)
		{
	                applet.init();
	                applet.start();
		}
		else System.out.println ("Program terminated.");
        }
        else System.out.println ("Wrong number of parameters. Use 'JRE STZ -?' or 'JAVA STZ -?' for help.");
}
}
